my $log = 'OpenILS::Utils::Logger';

#-------------------------------------------------------------------------------
package OpenILS::App::Storage::Pg::FTS;
use OpenILS::Utils::Logger qw/:level/;

sub compile {
	my $self = shift;
	my $class = ref($self) || $self;
	my $term = shift;
	my $joiner = shift || '&';

	$term =~ s/:/ /go;
	$term =~ s/(?:&[^;]+;)//go;
	$term =~ s/\s+/ /go;
	$term =~ s/(^|\s+)-(\w+)/$1!$2/go;
	$term =~ s/\b(\+)(\w+)/$2/go;
	$term =~ s/^\s*\b(.+)\b\s*$/$1/o;
	$term =~ s/^(?:an?|the)\b(.*)/$1/o;

	OpenILS::Utils::Logger->debug("Stripped search term string is [$term]",DEBUG);

	my @words = $term =~ /\b(\w+)\b/go;

	my @parts;
	while ($term =~ s/(['"\(]+)(.*?)((?<!\\)(?:\1|\))){1}//) {
		my $part = $2;
		$part =~ s/^\s*//o;
		$part =~ s/\s*$//o;
		next unless $part;
		push @parts, lc($part);
	}

	my @nots;
	while ($term =~ s/(!\w+)//) {
		push @nots, $1;
	}

	my $newterm = '';
	if (@nots) {
		$newterm .= '(';
	}
	$newterm .= join($joiner, grep($_ ? 1 : 0, split(/\s+/, $term)), (map { join($joiner, split(/\s+/) ) } @parts));

	if (@nots) {
		$newterm .= ')&('. join('&', @nots) . ')';
	}

	return bless { raw => $term, words => \@words, ts_query => $newterm, phrases => \@parts } => $class;
}

sub ts_query {
	my $self = shift;
	return $self->{ts_query};
}

sub raw {
	my $self = shift;
	return $self->{raw};
}

sub phrases {
	my $self = shift;
	return wantarray ? @{ $self->{phrases} } : $self->{phrases};
}

sub words {
	my $self = shift;
	return wantarray ? @{ $self->{words} } : $self->{words};
}

sub sql_phrases {
	my $self = shift;
	my $column = shift;
	my $output = '';
	for my $phrase ( $self->phrases ) {
		$phrase =~ s/%/\\%/go;
		$phrase =~ s/_/\\_/go;
		$phrase =~ s/'/\\_/go;
		$log->debug("Adding phrase [$phrase] to the match list", DEBUG);
		$output .= " AND $column ILIKE '\%$phrase\%'";
	}
	$log->debug("Phrase list is [$output]", DEBUG);
	return $output;
}

sub sql_word_bump {
	my $self = shift;
	my $column = shift;
	my $bump = shift || '0.1';
	my $output = '';
	for my $word ( $self->words ) {
		$word =~ s/%/\\%/go;
		$word =~ s/_/\\_/go;
		$word =~ s/'/''/go;
		$log->debug("Adding word [$word] to the relevancy bump list", DEBUG);
		$output .= " + CASE WHEN $column ILIKE '\%$word\%' THEN $bump ELSE 0 END";
	}
	$log->debug("Word bump list is [$output]", DEBUG);
	return $output;
}


#-------------------------------------------------------------------------------
use Class::DBI;

package Class::DBI;

{
	no warnings;
	no strict;
	sub _do_search {
		my ($proto, $search_type, @args) = @_;
		my $class = ref $proto || $proto;
		
		@args = %{ $args[0] } if ref $args[0] eq "HASH";

		my (@cols, @vals);
		my $search_opts = @args % 2 ? pop @args : {};

		$search_opts->{offset} = int($search_opts->{page}) * int($search_opts->{page_size})  if ($search_opts->{page_size});
		$search_opts->{_placeholder} ||= '?';

		while (my ($col, $val) = splice @args, 0, 2) {
			my $column = $class->find_column($col)
				|| (List::Util::first { $_->accessor eq $col } $class->columns)
				|| $class->_croak("$col is not a column of $class");

			push @cols, $column;
			push @vals, $class->_deflated_column($column, $val);
		}

		my $frag = join " AND ",
		map defined($vals[$_]) ? "$cols[$_] $search_type $$search_opts{_placeholder}" : "$cols[$_] IS NULL",
			0 .. $#cols;

		$frag .= " ORDER BY $search_opts->{order_by}"
			if $search_opts->{order_by};
		$frag .= " LIMIT $search_opts->{limit}"
			if $search_opts->{limit};
		$frag .= " OFFSET $search_opts->{offset}"
			if ($search_opts->{limit} && defined($search_opts->{offset}));

		return $class->sth_to_objects($class->sql_Retrieve($frag),
			[ grep defined, @vals ]);
	}
}

sub search_fti {
	my $self = shift;
	my @args = @_;
	if (ref($args[-1]) eq 'HASH') {
		$args[-1]->{_placeholder} = "to_tsquery('default',?)";
	} else {
		push @args, {_placeholder => "to_tsquery('default',?)"};
	}
	$self->_do_search("@@"  => @args);
}

#-------------------------------------------------------------------------------
package biblio::metarecord_field_entry;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'biblio.metarecord_field_entry' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Others => qw/record source field value value_fti/ );
__PACKAGE__->columns( TEMP => qw/headline rank/ );

__PACKAGE__->has_a( source => 'biblio::record' );
__PACKAGE__->has_a( record => 'biblio::metarecord' );
__PACKAGE__->has_a( field => 'biblio::metarecord_field_map' );

#__PACKAGE__->set_sql( by_record_with_headline => <<"SELECT" );
#		SELECT	*, headline(value,to_tsquery(?)) as headline
#		  FROM	biblio.metarecord_field_entry 
#		  WHERE	record = ?
#SELECT

sub search_by_record_with_headline {
	my $self = shift;
	my $rec = shift;
	my $ts_query = shift;

	my $select = <<"	SQL";
		SELECT *
		  FROM (
			SELECT	DISTINCT ON (field, value) id,record,field,value, headline(value,to_tsquery(?)) as headline
			  FROM	biblio.metarecord_field_entry 
			  WHERE	record = ?
		  ) t
		ORDER BY id;
	SQL
	
	my $sth = $self->db_Main->prepare($select);
	$sth->execute($ts_query,$rec);

	my (@rows, $h);
	push @rows, $h while ($h = $sth->fetchrow_hashref());

	return @rows;

	$log->debug("did we get a headline? $rows[0]{headline}", INTERNAL);

	return (map { $self->construct($_) } @rows );
}

sub field_class_entry_search {
	my $self = shift;
	my $field_class = shift;
	my $search = shift;
	my $distinct = shift || '';
	if ($distinct) {
		$distinct = "DISTINCT ON (e.$distinct)"
	}

	my $query = OpenILS::App::Storage::Pg::FTS->compile($search);
	my ($title_class) = biblio::metarecord_field_class_map->search( class => $field_class ) if ($field_class);

	my $ts_query = $query->ts_query;
	my $phrases = $query->sql_phrases('value');
	my $word_bump = $query->sql_word_bump('value');

	my $field_class_string = '';
	$field_class_string = 'AND m.field_class = '.$title_class if ($title_class);

	my $table = $self->table;
	my $select = <<"	SQL";
		SELECT * FROM (
			SELECT	$distinct e.id as id,e.record as record ,e.source as source,
				e.field as field,e.value as value,
				rank_cd(e.value_fti,to_tsquery(?)) * (1 $word_bump) as rank,
				headline(e.value,to_tsquery(?)) as headline
			  FROM	biblio.metarecord_field_entry e,
			  	biblio.metarecord_field_map m
			  WHERE	value_fti @@ to_tsquery(?) AND e.field = m.id
				$field_class_string
				$phrases
			  LIMIT 1000 ) t
		  
		  ORDER BY rank DESC, value ASC
	SQL

	$log->debug("Search SQL:\n$select",INTERNAL);

	my $sth = $self->db_Main->prepare($select);
	$sth->execute($ts_query,$ts_query,$ts_query);

	my (@rows, $h);
	push @rows, $h while ($h = $sth->fetchrow_hashref());

	return @rows;

	$log->debug("did we get a headline? $rows[0]{headline}", INTERNAL);

	return (map { $self->construct($_) } @rows );
}


#-------------------------------------------------------------------------------
package biblio::metarecord_field_map;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'biblio.metarecord_field_map' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Essential => qw/field field_class/ );
__PACKAGE__->columns( Others => qw/context_xpath extract_xpath
				   field_joiner importance
				   normalize_space lower
				   word_count canonical hitlist/ );

__PACKAGE__->has_a( field_class => 'biblio::metarecord_field_class_map' );
__PACKAGE__->has_many( field_entries => 'biblio::metarecord_field_entry' );

#-------------------------------------------------------------------------------
package biblio::metarecord_field_class_map;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'biblio.metarecord_field_class_map' );
__PACKAGE__->columns( All => qw/id class/ );

__PACKAGE__->has_many( field_maps => 'biblio::metarecord_field_map' );

#-------------------------------------------------------------------------------
package biblio::metarecord;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'biblio.metarecord' );
__PACKAGE__->sequence( 'biblio.metarecord_id_seq' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Essential => qw/fingerprint opac_visible/ );

__PACKAGE__->has_many( field_entries => 'biblio::metarecord_field_entry' );
__PACKAGE__->has_many( records => 'biblio::record' );

#-------------------------------------------------------------------------------
package asset::call_number;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'asset.call_number' );
__PACKAGE__->sequence( 'asset.call_number_id_seq' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Essential => qw/record copycount/ );

__PACKAGE__->has_a( record => 'biblio::record' );
__PACKAGE__->has_many( copies => 'asset::copy' );



#-------------------------------------------------------------------------------
package asset::copy;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'asset.copy' );
__PACKAGE__->sequence( 'asset.copy_id_seq' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Essential => qw/call_number barcode/ );

__PACKAGE__->has_a( call_number => 'asset::call_number' );



#-------------------------------------------------------------------------------
package biblio::record;
use base qw/OpenILS::App::Storage::CDBI/;

__PACKAGE__->table( 'biblio.record' );
__PACKAGE__->sequence( 'biblio.record_id_seq' );
__PACKAGE__->columns( Primary => qw/id/ );
__PACKAGE__->columns( Essential => qw/control brief_title metarecord copycount/ );
__PACKAGE__->columns( Others => qw/datum/ );
__PACKAGE__->columns( TEMP => qw/rank headline/ );

__PACKAGE__->has_a( metarecord => 'biblio::metarecord' );
__PACKAGE__->has_many( field_entries => 'biblio::metarecord_field_entry' );
__PACKAGE__->has_many( call_numbers => 'asset::call_number' );

#-------------------------------------------------------------------------------
package OpenILS::App::Storage::Pg;
use base qw/Class::DBI OpenILS::App::Storage/;
use DBI;
use OpenILS::DomainObject::oilsSearch;
use OpenILS::Utils::Logger qw/:level/;
use OpenILS::Utils::Config;
use XML::LibXSLT;

my $xslt_parser = XML::LibXSLT->new;
my $xml_parser = OpenILS::DOM->new;

__PACKAGE__->set_sql( retrieve_limited => 'SELECT * FROM __TABLE__ ORDER BY id LIMIT ?' );

my $_dbh;

sub db_Main {	
	return $_dbh if (eval { $_dbh->selectrow_arrayref('select 1')->[0] });

	my $self = shift;
	my %args = (%OpenILS::App::Storage::_db_params,@_);

	my %attrs = (	%{$self->_default_attributes},
			RootClass => 'DBIx::ContextualFetch',
			ShowErrorStatement => 1,
			RaiseError => 1,
			AutoCommit => 1,
			PrintError => 1,
			Taint => 1,
			pg_enable_utf8 => 1,
			FetchHashKeyName => 'NAME_lc',
			ChopBlanks => 1,
	);

	$log->debug(" Default attributes for this DB connection are:\n\t".join("\n\t",map { "$_\t==> $attrs{$_}" } keys %attrs), INTERNAL);

	$_dbh = DBI->connect( "dbi:Pg:host=$args{host};dbname=$args{database}",$args{user},$args{pw}, \%attrs );
	$_dbh->do("SET CLIENT_ENCODING TO 'SQL_ASCII';");

	return $_dbh;
}

sub storage_general_retrieve_limited {
	my $client = shift;
	my $search = shift;

	my $table = $search->class;
	$log->debug("storage_general_retrieve_limited using CDBI class $table", DEBUG);
	
	my $sql = $search->toSQL;
	$log->debug("Generated SQL [ $sql ]", DEBUG);

	my $sth = $table->db_Main->prepare( $search->toSQL );
	$sth->execute;

	#while ( my $row = $table->construct( $sth->fetchrow_hashref ) ) {
	#	$client->respond( _cdbi2oilsHash( $row, $search->fields ) );
	#}

	while ( my $row = $table->construct( $sth->fetchrow_hashref ) ) {
		$client->respond( _hashref2oilsHash( $row, $search->fields ) );
	}

	return undef;
}

sub storage_opac_hitlist_count {
	my $client = shift;
	my $type = shift;
	my $search = shift;
	my $reg = shift;
	my $lib = shift || '%';

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);


	throw OpenILS::EX::InvalidArg ("No search specified! (type is $type)") unless ($search);

	my $column = 'value_fti';
	my $field_class_string = '';
	my ($title_class) = biblio::metarecord_field_class_map->search( class => $type );
	$field_class_string = 'AND m.field_class = '.$title_class;

	my $marc = '';
	if ($search =~ /^marc:(\w+)/oi) {
		$marc = $1;
		$log->debug("MARC search [$marc]",DEBUG);
		$search =~ s/^marc:\w+\s*(.+)$/$1/o;
		$column = 'index_vector';
	}



	my $query = OpenILS::App::Storage::Pg::FTS->compile($search);

	($type) = __PACKAGE__->db_Main->selectrow_array(<<"	SQL");
		SELECT func.biblio_metarecord_field_class_map('$type');
	SQL

	my $ts_query = $query->ts_query;

	my $select;
	if ($marc) {
		my $field = $marc;
		my $subfield = '';
		if ($field =~ /^(\d+)([a-z]+?)$/io) {
			$field = $1;
			$subfield = $2;
		}

		$marc = "AND tag = '$field' ";
		$marc .= "AND subfield = '$subfield'" if (defined $subfield);
	
		my $phrases = $query->sql_phrases('data');

		$select = <<"		SQL";
			SELECT	count(distinct e.record),
				count(e.id)
			  FROM	biblio.metarecord_field_entry e,
			  	asset.call_number c,
			  	biblio.full_record_metadata r,
			  	biblio.metarecord_field_map m
			  WHERE	$column @@ to_tsquery(?)
			  	$marc
			 	AND e.field = m.id
			    	AND r.record = c.record
				AND c.record = e.source
				$locality
				$phrases
		SQL
	} else {
		my $phrases = $query->sql_phrases('value');

		$select = <<"		SQL"
			SELECT count(distinct record), count(id) FROM (
				SELECT	e.id as id, e.record as record,
					m.field_class as field_class
				  FROM	biblio.metarecord_field_entry e
					LEFT JOIN biblio.metarecord_field_map m ON (m.id = e.field)
					JOIN biblio.metarecord_field_class_map cl ON (cl.id = m.field_class)
					JOIN asset.call_number c ON (c.record = e.source)
				  WHERE	value_fti @@ to_tsquery(?)
					$locality
					$phrases
				LIMIT 100000
			) t
			WHERE field_class = ?
		SQL


		#$select = <<"		SQL";
		#	SELECT	count(distinct e.record)
		#	  FROM	biblio.metarecord_field_entry e,
		#	  	asset.call_number c,
		#	  	biblio.record r,
		#	  	biblio.metarecord_field_map m
		#	  WHERE	$column @@ to_tsquery(?)
		#	 	AND e.field = m.id
		#	    	AND r.id = c.record
		#		AND c.record = e.source
		#		$field_class_string
		#		$locality
		#		$phrases
		#SQL
	}

	$log->debug("Search SQL:\n$select",INTERNAL);

	my $sth = __PACKAGE__->db_Main->prepare($select);
	$sth->execute($ts_query) if ($marc);
	$sth->execute($ts_query, $title_class->id) unless ($marc);

	return new OpenILS::DomainObjectCollection::oilsArray ($sth->fetchrow_array);

}

sub storage_opac_metarecord_copy_count {
	my $client = shift;
	my $mr = shift;
	my $reg = shift;
	my $lib = shift || '%';

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);


	throw OpenILS::EX::InvalidArg ("No Metarecord specified!") unless ($mr =~ /^\d+$/o);

	my ($count) = biblio::record->db_Main->selectrow_array(<<"	SQL");
		SELECT	sum(c.copycount)
		  FROM	asset.call_number c,
		  	biblio.record r
		  WHERE	r.metarecord = $mr
		  	AND c.record = r.id
			$locality;
	SQL

	return new OpenILS::DomainObject::oilsScalar ($count);
}

sub storage_opac_record_copy_count {
	my $client = shift;
	my $rec = shift;
	my $reg = shift;
	my $lib = shift || '%';

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);


	throw OpenILS::EX::InvalidArg ("No Record specified!") unless ($rec =~ /^\d+$/o);

	my ($count) = biblio::record->db_Main->selectrow_array(<<"	SQL");
		SELECT	sum(c.copycount)
		  FROM	asset.call_number c,
		  	biblio.record r
		  WHERE	r.id = $rec
		  	AND c.record = r.id
			$locality
	SQL

	return new OpenILS::DomainObject::oilsScalar ($count);
}

sub storage_opac_metarecord_avail_count {
	my $client = shift;
	my $mr = shift;
	my $reg = shift;
	my $lib = shift || '%';

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);


	throw OpenILS::EX::InvalidArg ("No Metarecord specified!") unless ($mr =~ /^\d+$/o);

	my ($count) = biblio::record->db_Main->selectrow_array(<<"	SQL");
		SELECT	count(distinct p.id)
		  FROM	asset.call_number c,
		  	asset.copy p,
		  	biblio.record r
		  WHERE	r.metarecord = $mr
		  	AND c.record = r.id
			AND p.call_number = c.id
			AND p.current_lib = ''
			$locality;
	SQL

	return new OpenILS::DomainObject::oilsScalar ($count);
}

sub storage_opac_record_avail_count {
	my $client = shift;
	my $rec = shift;
	my $reg = shift;
	my $lib = shift || '%';

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);


	throw OpenILS::EX::InvalidArg ("No Record specified!") unless ($rec =~ /^\d+$/o);

	my ($count) = biblio::record->db_Main->selectrow_array(<<"	SQL");
		SELECT	count(distinct p.id)
		  FROM	asset.call_number c,
		  	asset.copy p,
		  	biblio.record r
		  WHERE	r.id = $rec
		  	AND c.record = r.id
			AND p.call_number = c.id
			AND p.current_lib = ''
			$locality
	SQL

	return new OpenILS::DomainObject::oilsScalar ($count);
}


sub storage_opac_MODS_from_metarecord {
	my $client = shift;
	my $mr = shift;

	throw OpenILS::EX::InvalidArg ("No Metarecord specified!") unless ($mr);

	my $sd = $xml_parser->parse_file(OpenILS::Utils::Config->current->dirs->xslt_dir . '/MARC21slim2MODS.xsl');
	my $ss = $xslt_parser->parse_stylesheet($sd);
	my @recs = biblio::metarecord->retrieve($mr)->records;
	$log->debug("storage_opac_MODS_from_metarecord got ".scalar(@recs)." records.",DEBUG);
	for my $record ( @recs ) {
		$log->debug("XSLT: Transforming:\n".$record->datum,DEBUG);
		
		my $doc = $xml_parser->parse_string($record->datum);
		my $res = $ss->transform($doc)->documentElement;
		my $mods = $xml_parser->parse_string( $ss->output_string($res) )->documentElement;
		
		$log->debug("Created document:\n".$mods->toString(1), INTERNAL);
		$client->respond( new OpenILS::DomainObject::oilsScalar ($mods) );
	}
	return undef;
}

sub storage_opac_hitlist {
	my $client = shift;
	my $type = shift;
	my $search = shift;
	my $limit = shift || 10;
	my $offset = shift || 0;
	my $reg = shift;
	my $lib = shift || '%';

	throw OpenILS::EX::InvalidArg ("No type specified!") unless ($type);
	throw OpenILS::EX::InvalidArg ("No search specified! (type is $type)") unless ($search);

	##my @urecords = map { "".$_->record if ($_) } biblio::metarecord_field_entry->field_class_entry_search($type,$search,'record');
	#my @urecords = biblio::metarecord_field_entry->field_class_entry_search($type,$search,'record',$limit,$offset);

	##my @urecords;
	##for my $record ( @records ) {
	##	push @urecords, $record unless (grep { $record == $_ } @urecords);
	##}

	#@urecords = @urecords[$offset .. int($offset + $limit - 1)];

	my $column = 'value_fti';
	my $field_class_string = '';

	my ($title_class) = biblio::metarecord_field_class_map->search( class => $type );

	$field_class_string = 'AND m.field_class = '.$title_class;

	my $marc = '';
	if ($search =~ /^marc:(\w+)/oi) {
		$marc = $1;
		$log->debug("MARC search [$marc]",DEBUG);
		$search =~ s/^marc:\w+\s*(.+)$/$1/o;
		$column = 'index_vector';
	}

	my $query = OpenILS::App::Storage::Pg::FTS->compile($search);

	my $locality = '';
	$locality = "$reg-$lib" if ($reg);
	$locality = "%" unless ($reg);

	my $locality_clause = '';
	$locality_clause = "AND c.owning_lib LIKE '$locality'" if ($reg);
	$log->debug("Locality phrase is [$locality_clause]",DEBUG);

	my $ts_query = $query->ts_query;

	my $select;
	if ($marc) {
		my $field = $marc;
		my $subfield = '';
		if ($field =~ /^(\d+)([a-z]+?)$/io) {
			$field = $1;
			$subfield = $2;
		}



		$marc = "AND tag = '$field' ";
		$marc .= "AND subfield = '$subfield'" if (defined $subfield);
		
		my $phrases = $query->sql_phrases('data');
		my $word_bump = $query->sql_word_bump('data');

		my ($first_word) = grep { $_ !~ /^(?:an?|the)$/ } $query->words;
		$first_word = lc $first_word;
		my $begin_bonus = "( CASE WHEN POSITION('$first_word' IN LOWER(r.data)) = 1 THEN 0.1 ELSE 0 END )"; 

		$select = <<"		SQL";
		  	SELECT		sum(rank_cd($column,to_tsquery(?)) * (1 $word_bump) + $begin_bonus) as rank,
		  			e.record as record
			  FROM		biblio.metarecord_field_entry e,
				  	asset.call_number c,
				  	biblio.full_record_metadata r,
				  	biblio.metarecord_field_map m
		  	  WHERE		$column @@ to_tsquery(?)
			  		$marc
		    			AND e.field = m.id
					AND c.record = e.source
		    			AND r.record = c.record
					$locality_clause
					$phrases
		    	GROUP BY	2
		    	ORDER BY	1 DESC
		    	LIMIT		$limit
		    	OFFSET		$offset
		SQL
	} else {

		#$select = <<"		SQL"
		#	SELECT	rank, record
		#	  FROM	func.hitlist_base(?,?)
		#	  WHERE	field_class = ?
		#	  ORDER BY 1 DESC
		#	  LIMIT $limit OFFSET $offset
		#SQL

		my $phrases = $query->sql_phrases('value');

		$select = <<"		SQL"
			SELECT * FROM (
				SELECT	sum(rank_cd(value_fti,to_tsquery(?)) *
						( 1 + CASE WHEN POSITION(split_part(?, '&', 1) IN LOWER(e.value)) = 1 THEN 0.1 ELSE 0 END +
						func.match_count(?, e.value)))::NUMERIC as rank,
					e.record as record,
					m.field_class as field_class
				  FROM	biblio.metarecord_field_entry e
					LEFT JOIN biblio.metarecord_field_map m ON (m.id = e.field)
					JOIN biblio.metarecord_field_class_map cl ON (cl.id = m.field_class)
					JOIN asset.call_number c ON (c.record = e.source)
				  WHERE	value_fti @@ to_tsquery(?)
					$locality_clause
					$phrases
				GROUP BY 2,3
				LIMIT 100000
			) t
			WHERE field_class = ?
			ORDER BY 1 DESC
			LIMIT $limit OFFSET $offset;
		SQL

	
		#my $word_bump = $query->sql_word_bump('value');

		#my ($first_word) = grep { $_ !~ /^(?:an?|the)$/ } $query->words;
		#$first_word = lc $first_word;
		#my $begin_bonus = "( CASE WHEN POSITION('$first_word' IN LOWER(e.value)) = 1 THEN 0.1 ELSE 0 END )"; 

		#$select = <<"		SQL";
		#  	SELECT		sum(rank_cd($column,to_tsquery(?)) * (1 $word_bump) + $begin_bonus) as rank,
		#  			e.record as record
		#	  FROM		biblio.metarecord_field_entry e,
		#		  	asset.call_number c,
		#		  	biblio.record r,
		#		  	biblio.metarecord_field_map m
		#  	  WHERE		$column @@ to_tsquery(?)
		#    			AND e.field = m.id
		#			AND c.record = e.source
		#    			AND r.id = c.record
		#			$locality
		#			$field_class_string
		#			$phrases
		#    	GROUP BY	2
		#    	ORDER BY	1 DESC
		#    	LIMIT		$limit
		#    	OFFSET		$offset
		#SQL
	}

	$log->debug("Search SQL:\n$select",INTERNAL);

	my $sth = __PACKAGE__->db_Main->prepare($select);
	$sth->execute($ts_query,$ts_query) if ($marc);
	#$sth->execute($ts_query,$locality,$title_class) if (!$marc);
	$sth->execute($ts_query,$ts_query,$ts_query,$ts_query,$title_class->id) if (!$marc);

	my $mr_select = <<"	SQL";
		SELECT *
		  FROM (
			SELECT	DISTINCT ON (e.field, e.value) e.id,e.record,e.field,m.field_class,e.value, headline(value,to_tsquery(?)) as headline, r.copycount
			  FROM	biblio.metarecord_field_entry e, 
			  	biblio.metarecord_field_map m,
			  	biblio.record r
			  WHERE	e.record = ?
	    			AND m.hitlist IS TRUE
			  	AND m.id = e.field
			  	AND e.source = r.id
		  ) t
		ORDER BY id
	SQL
	
	my $mr_sth = __PACKAGE__->db_Main->prepare($mr_select);


	while ( my $record = $sth->fetchrow_hashref()) {
		next unless ($record);
		$mr_sth->execute($ts_query,$record->{record});
		my $hitlist = new OpenILS::DomainObjectCollection::oilsArray;
		#$hitlist->push( _cdbi2oilsHash( $_ ) )
		#	for ( biblio::metarecord_field_entry->search_by_record_with_headline( $record, $query->ts_query) );
		#$hitlist->push( _hashref2oilsHash( $_ ) )
		#	for ( biblio::metarecord_field_entry->search_by_record_with_headline( $record->{record}, $query->ts_query) );
		while ( my $href = $mr_sth->fetchrow_hashref() ) {
			$hitlist->push( _hashref2oilsHash( $href ) );
		}
		$client->respond( $hitlist );
	}

	return undef;
}

sub storage_opac_hitlist_nosort {
	my $client = shift;
	my $type = shift;
	my $search = shift;
	my $limit = shift || 10;
	my $offset = shift || 0;
	my $reg = shift;
	my $lib = shift || '%';

	throw OpenILS::EX::InvalidArg ("No type specified!") unless ($type);
	throw OpenILS::EX::InvalidArg ("No search specified! (type is $type)") unless ($search);

	##my @urecords = map { "".$_->record if ($_) } biblio::metarecord_field_entry->field_class_entry_search($type,$search,'record');
	#my @urecords = biblio::metarecord_field_entry->field_class_entry_search($type,$search,'record',$limit,$offset);

	##my @urecords;
	##for my $record ( @records ) {
	##	push @urecords, $record unless (grep { $record == $_ } @urecords);
	##}

	#@urecords = @urecords[$offset .. int($offset + $limit - 1)];

	my $column = 'value_fti';
	my $field_class_string = '';

	my ($title_class) = biblio::metarecord_field_class_map->search( class => $type );

	$field_class_string = 'AND m.field_class = '.$title_class;

	my $marc = '';
	if ($search =~ /^marc:(\w+)/oi) {
		$marc = $1;
		$log->debug("MARC search [$marc]",DEBUG);
		$search =~ s/^marc:\w+\s*(.+)$/$1/o;
		$column = 'index_vector';
	}

	my $query = OpenILS::App::Storage::Pg::FTS->compile($search);

	my $locality = '';
	$locality = "$reg-$lib" if ($reg);
	$locality = "%" unless ($reg);

	my $locality_clause = '';
	$locality_clause = "AND c.owning_lib LIKE '$locality'" if ($reg);
	$log->debug("Locality phrase is [$locality_clause]",DEBUG);

	my $ts_query = $query->ts_query;

	my $select;
	if ($marc) {
		my $field = $marc;
		my $subfield = '';
		if ($field =~ /^(\d+)([a-z]+?)$/io) {
			$field = $1;
			$subfield = $2;
		}



		$marc = "AND tag = '$field' ";
		$marc .= "AND subfield = '$subfield'" if (defined $subfield);
		
		my $phrases = $query->sql_phrases('data');

		$select = <<"		SQL";
		  	SELECT DISTINCT	e.record as record
			  FROM		biblio.metarecord_field_entry e,
				  	asset.call_number c,
				  	biblio.full_record_metadata r,
				  	biblio.metarecord_field_map m
		  	  WHERE		$column @@ to_tsquery(?)
			  		$marc
		    			AND e.field = m.id
					AND c.record = e.source
		    			AND r.record = c.record
					$locality_clause
					$phrases
		    	LIMIT		$limit
		    	OFFSET		$offset
		SQL
	} else {

		#$select = <<"		SQL"
		#	SELECT	rank, record
		#	  FROM	func.hitlist_base(?,?)
		#	  WHERE	field_class = ?
		#	  ORDER BY 1 DESC
		#	  LIMIT $limit OFFSET $offset
		#SQL

		my $phrases = $query->sql_phrases('value');

		$select = <<"		SQL"
			SELECT	DISTINCT e.record as record
			  FROM	biblio.metarecord_field_entry e,
				biblio.metarecord_field_map m, 
				asset.call_number c
			  WHERE	e.value_fti @@ to_tsquery(?)
				AND m.id = e.field
				AND c.record = e.source
				AND m.field_class = ?
				$locality_clause
				$phrases
			LIMIT $limit OFFSET $offset;
		SQL

	
		#my $word_bump = $query->sql_word_bump('value');

		#my ($first_word) = grep { $_ !~ /^(?:an?|the)$/ } $query->words;
		#$first_word = lc $first_word;
		#my $begin_bonus = "( CASE WHEN POSITION('$first_word' IN LOWER(e.value)) = 1 THEN 0.1 ELSE 0 END )"; 

		#$select = <<"		SQL";
		#  	SELECT		sum(rank_cd($column,to_tsquery(?)) * (1 $word_bump) + $begin_bonus) as rank,
		#  			e.record as record
		#	  FROM		biblio.metarecord_field_entry e,
		#		  	asset.call_number c,
		#		  	biblio.record r,
		#		  	biblio.metarecord_field_map m
		#  	  WHERE		$column @@ to_tsquery(?)
		#    			AND e.field = m.id
		#			AND c.record = e.source
		#    			AND r.id = c.record
		#			$locality
		#			$field_class_string
		#			$phrases
		#    	GROUP BY	2
		#    	ORDER BY	1 DESC
		#    	LIMIT		$limit
		#    	OFFSET		$offset
		#SQL
	}

	$log->debug("Search SQL:\n$select",INTERNAL);

	my $sth = __PACKAGE__->db_Main->prepare($select);
	$sth->execute($ts_query,$ts_query) if ($marc);
	#$sth->execute($ts_query,$locality,$title_class) if (!$marc);
	$sth->execute($ts_query,$title_class->id) if (!$marc);

	my $mr_select = <<"	SQL";
		SELECT *
		  FROM (
			SELECT	DISTINCT ON (e.field, e.value) e.id,e.record,e.field,m.field_class,e.value, headline(value,to_tsquery(?)) as headline, r.copycount
			  FROM	biblio.metarecord_field_entry e, 
			  	biblio.metarecord_field_map m,
			  	biblio.record r
			  WHERE	e.record = ?
			  	AND m.id = e.field
			  	AND e.source = r.id
		  ) t
		ORDER BY id
	SQL
	
	my $mr_sth = __PACKAGE__->db_Main->prepare($mr_select);


	while ( my $record = $sth->fetchrow_hashref()) {
		next unless ($record);
		$mr_sth->execute($ts_query,$record->{record});
		my $hitlist = new OpenILS::DomainObjectCollection::oilsArray;
		#$hitlist->push( _cdbi2oilsHash( $_ ) )
		#	for ( biblio::metarecord_field_entry->search_by_record_with_headline( $record, $query->ts_query) );
		#$hitlist->push( _hashref2oilsHash( $_ ) )
		#	for ( biblio::metarecord_field_entry->search_by_record_with_headline( $record->{record}, $query->ts_query) );
		while ( my $href = $mr_sth->fetchrow_hashref() ) {
			$hitlist->push( _hashref2oilsHash( $href ) );
		}
		$client->respond( $hitlist );
	}

	return undef;
}

sub storage_opac_recordlist_from_metarecord {
	my $client = shift;
	my $mr = shift;
	my $reg = shift;
	my $lib = shift || '%';

	throw OpenILS::EX::InvalidArg ("No metarecord specified!") unless ($mr =~ /^\d+$/);

	my $locality = '';
	$locality = "AND c.owning_lib LIKE '$reg-$lib'" if ($reg);
	$log->debug("Locality phrase is [$locality]",DEBUG);

	my $select = <<"	SQL";
		SELECT  t.id FROM (
			SELECT	DISTINCT r.id, r.copycount
			  FROM	biblio.record r,
			  	asset.call_number c
			  WHERE	r.metarecord = ?
			  	AND r.id = c.record
				$locality
			) t
				
		  ORDER BY t.copycount DESC
	SQL

	my $sources = __PACKAGE__->db_Main->selectcol_arrayref($select,{},$mr);

	my $pselect = <<"	SQL";
		SELECT	DISTINCT ON (e.field, e.value) e.id,e.record,e.source,e.field,m.field_class,e.value
		  FROM	biblio.metarecord_field_entry e,
			  biblio.metarecord_field_map m
		  WHERE	e.source = ?
		  	AND m.id = e.field
	SQL


	my $psth = __PACKAGE__->db_Main->prepare($pselect);

	for my $record (@$sources) {
		next unless ($record);
		$log->debug("Got metarecord $record",INTERNAL);
		my $hitlist = new OpenILS::DomainObjectCollection::oilsArray;
		$psth->execute($record);
		while (my $h = $psth->fetchrow_hashref) {
			$log->debug("Got metarecord entry $$h{id} ($$h{value})",INTERNAL);
			$hitlist->push( _hashref2oilsHash( $h ) );
		}
		$client->respond( $hitlist );
	}

	return undef;
}

sub storage_opac_callnumber_list_from_record {
	my $client = shift;
	my $rec = shift;

	throw OpenILS::EX::InvalidArg ("No record specified!") unless ($rec =~ /^\d+$/);

	my $select = <<"	SQL";
		SELECT	n.copycount, n.label, n.owning_lib, count(distinct c.id) as available
		  FROM	asset.call_number n
			LEFT OUTER JOIN asset.copy c ON (c.call_number = n.id AND c.current_lib = '')
		  WHERE	n.record = ?
		  GROUP BY 1,2,3
	SQL

	my $sth = __PACKAGE__->db_Main->prepare($select);

	$sth->execute($rec);
	
	my @rows;
	while (my $h = $sth->fetchrow_hashref) {
		push @rows, $h;
	}
	$client->respond( _hashref_list2oilsArray_of_oilsHash( @rows ) );


	return undef;
}

sub storage_opac_callnumber_list_from_metarecord {
	my $client = shift;
	my $rec = shift;
	my $reg = shift || '';
	my $lib = shift || '';

	throw OpenILS::EX::InvalidArg ("No record specified!") unless ($rec =~ /^\d+$/);

	my $select = <<"	SQL";
		SELECT	n.copycount, n.label, n.owning_lib, count(distinct c.id) as available
		  FROM	(asset.call_number n
		  	 JOIN biblio.metarecord_field_entry e ON (n.record = e.source))
			LEFT OUTER JOIN asset.copy c ON (c.call_number = n.id AND c.current_lib = '')
		  WHERE	e.record = ?
		  GROUP BY 1,2,3
	SQL

	$select = <<"	SQL";
		SELECT	distinct n.id, n.copycount, n.label, n.owning_lib
		  FROM	asset.call_number n,
		  	biblio.metarecord_field_entry e
		  WHERE	n.record = e.source
		  	AND e.record = ?
	SQL

	my $sth = __PACKAGE__->db_Main->prepare($select);

	$sth->execute($rec);
	
	my @rows;

	my %count = (	local_avail	=> 0,	local_total	=> 0,
			reg_avail	=> 0,	reg_total	=> 0,
			all_avail	=> 0,	all_total	=> 0 );
	
	my $lsth = __PACKAGE__->db_Main->prepare("SELECT count( id ) FROM asset.copy WHERE call_number = ? AND current_lib = ''");
	while (my $h = $sth->fetchrow_hashref) {
		$lsth->execute($$h{id});
		my ($loc) = $lsth->fetchrow_array;
		$lsth->finish;
		#my $loc = __PACKAGE__->db_Main->selectrow_arrayref("SELECT count( distinct id ) FROM asset.copy WHERE call_number = $$h{id} AND current_lib = ''")->[0];
		if ($lib && $$h{owning_lib} eq "$reg-$lib") {
			$count{local_avail} += $loc;
			$count{local_total} += $$h{copycount};
			$count{reg_avail} += $loc;
			$count{reg_total} += $$h{copycount};
		} elsif ($reg && $$h{owning_lib} =~ /^$reg-/) {
			$count{reg_avail} += $loc;
			$count{reg_total} += $$h{copycount};
		}
		$count{all_avail} += $loc;
		$count{all_total} += $$h{copycount};
	}
	$sth->finish;
	return _hashref2oilsHash( \%count );
}

sub _cdbi2oilsHash {
	my $obj = shift;
	my @cols = @_;
	unless (@cols) {
		@cols = ($obj->columns('All'), $obj->columns('TEMP'));
	}
	return new OpenILS::DomainObjectCollection::oilsHash ( map { ( $_ => $obj->{$_} ) } @cols );
}

sub _hashref2oilsHash {
	my $obj = shift;
	return new OpenILS::DomainObjectCollection::oilsHash ( %$obj );
}

sub _cdbi_list2oilsArray_of_oilsHash {
	my @objs = @_;
	return new OpenILS::DomainObjectCollection::oilsArray ( map { _cdbi2oilsHash($_) } @objs );
}

sub _hashref_list2oilsArray_of_oilsHash {
	my @objs = @_;
	return new OpenILS::DomainObjectCollection::oilsArray ( map { _hashref2oilsHash($_) } @objs );
}

1;
