package OpenILS::Transport::SlimJabber::Inbound;
use strict;use warnings;
use base qw/OpenILS::Transport::SlimJabber::Client/;
use OpenILS::EX;
use OpenILS::Utils::Config;
use OpenILS::Utils::Logger qw(:level);

my $logger = "OpenILS::Utils::Logger";

=head1 Description

This is the jabber connection where all incoming client requests will be accepted.
This connection takes the data, passes it off to the system then returns to take
more data.  Connection params are all taken from the config file and the values
retreived are based on the $app name passed into new().

This service should be loaded at system startup.

=cut

# XXX This will be overhauled to connect as a component instead of as
# a user.  all in good time, though.

{
	my $unix_sock;
	sub unix_sock { return $unix_sock; }
	my $instance;

	sub new {
		my( $class, $app ) = @_;
		$class = ref( $class ) || $class;
		if( ! $instance ) {
			my $app_state = $app . "_inbound";
			my $config = OpenILS::Utils::Config->current;

			if( ! $config ) {
				throw OpenILS::EX::Jabber( "No suitable config found" );
			}

			my $username	= $config->transport->users->$app;
			my $password	= $config->transport->auth->password;
			my $resource	= "system";


			my $self = __PACKAGE__->SUPER::new( 
					username		=> $username,
					resource		=> $resource,
					password		=> $password,
					);
					
					
			my $f = $config->dirs->sock_dir;
			$unix_sock = join( "/", $f, $config->unix_sock->$app );
			bless( $self, $class );
			$instance = $self;
		}
		return $instance;
	}

}
	
sub listen {
	my $self = shift;

	
	while(1) {
		my $sock = $self->unix_sock();
		my $socket = IO::Socket::UNIX->new( Peer => $sock  );
	
		throw OpenILS::EX::Socket( "Unable to connect to UnixServer: socket-file: $sock \n :=> $! " )
			unless ($socket->connected);

		my $o = $self->process( -1 );

		if( ! defined( $o ) ) {
			throw OpenILS::EX::Jabber( "Listen Loop failed at 'process()'" );
		}
		print $socket $o;

		$socket->close;

	}

	throw OpenILS::EX::Socket( "How did we get here?!?!" );
}

1;

