package OpenILS::Utils::JabberUtils;
use strict; use warnings;
use base qw( OpenILS );
use Net::Jabber qw( Client );
use XML::LibXML;


=head1 Description

OpenILS::Utils::JabberUtils

A Set of tools that may be used at runtime to determine information about
the jabber server

=cut



sub online_users {

	my @online_users;
	my $config = OpenILS::Utils::Config->current;

	my $host			= $config->transport->server->primary;
	my $username	= $config->transport->users->admin;
	my $password	= $config->transport->auth->password;
	my $resource	= "monitor";

	my $connection = Net::Jabber::Client->new();
	$connection->Connect( 'hostname' => $host );
	my @a = $connection->AuthSend( 'username' => $username, 
		'password' => $password, 'resource' => $resource );

	if( $a[0] ne "ok" ) {
		return ( "Jabber connection not OK: @a" );
	}

	# --- Handle response from server
	$connection->SetCallBacks('iq' => sub {
		my( $id, $iq ) = @_;
		my $xml =  $iq->GetXML();
		my $parser = XML::LibXML->new();
		my $doc = $parser->parse_string( $xml );
		my @nodes = $doc->getDocumentElement->findnodes( "/*/*/*[local-name()='item']" );
		use Data::Dumper;
		foreach my $n (@nodes ) {
			foreach my $attr( $n->attributes() ) {
				if( $attr->nodeName eq "jid" ) {
					push( @online_users, $attr->value );
					#print $attr->value . "\n";
				}
			}
		}
		$connection->Disconnect();
	} );
	# -----

	my $iq = Net::Jabber::IQ->new();
	$iq->SetTo( $host );
	$iq->SetFrom( "$username\@$host\/$resource" );
	$iq->SetType( "get" );
	$iq->SetID( "1232123" );
	$iq->InsertRawXML( "<query node='sessions' xmlns='http://jabber.org/protocol/disco#items'/>" );
	$connection->Send( $iq );

	while( 1 ) {
		my $o = $connection->Process();
		if( ! defined( $o ) ) {
			last;
		}
	}

	return @online_users;
}

1;

