package OpenILS::Utils::NetLogger;
use strict;
use base qw(OpenILS::Utils);
use IO::Socket::INET;
use OpenILS::Utils::LogServer;
use OpenILS::Utils::Logger;

=head1 Name 

OpenILS::Utils::NetLogger

=cut

=head1 Synopsis

	NetLogger->log( $logfile, $loglevel, $message );

=cut

=head1 Description

NetLogger provides a logging interface which takes log requests and sends them to a log server .  

=cut
sub LOGSERVER {"OpenILS::Utils::LogServer"}

use constant PORT			=> LOGSERVER->PORT;
use constant PROTO		=> LOGSERVER->PROTO;
use constant PEER			=> 'localhost';

	
=head2 log( $logfile, $level, @messages )

This is where the work is done.  Provide a log file name, a log level
and one or more messages (in an array if it's more than one) and this method
will format and send the data to the log server. 

The protocol is pretty simple:
$udp_packet = $logname|$loglevel|$message

=cut 
sub log {
	my( $class, $file, $level, @messages ) = @_;

	my $port		= PORT;
	my $proto	= PROTO;
	my $peer		= PEER;

	
	my $socket = IO::Socket::INET->new(
			PeerAddr	=> $peer,
			PeerPort	=> $port,
			Proto		=> $proto )
		or die "Unable to open socket to log server";  # throw locally logged exception

	foreach my $msg (@messages) {
		my $packet = "$file|$level|$msg";
		$socket->send( $packet );
	}

	close( $socket );
}

1;
	

